// All Rights Reserved, Copyright (c) FUJITSU LIMITED 2002
#include    <stdio.h>
#include    <stdlib.h>
#include    <time.h>
#include    "orb_cplus.h"
#include    "CosNaming_cplus.h"
#include    "EventService_cplus.h"

#define UNKNOWN_PROXY "UnknownProxy"

#if defined(_MSC_VER) && (_MSC_VER >= 1400)
char TimeBuffer[27];
#else
char *TimeBuffer = 0;
#endif

// ---- get Access-Time -------------------------------------- */
static void str_time( time_t clock )
{
#if defined(_MSC_VER) && (_MSC_VER >= 1400)
    memset( TimeBuffer, 0x00, sizeof(TimeBuffer) );
    ctime_s( TimeBuffer, sizeof(TimeBuffer), &clock );
#else
    TimeBuffer = ctime( &clock );
#endif
    return;
}

// ---- get IP-Address -------------------------------------------
static void
str_ipaddress(
    CORBA::Long    ip,
    char    *buffer )
{
    CORBA::Long    tmp[4];

    tmp[0] = ( ip >> 0  ) & 0xff;
    tmp[1] = ( ip >> 8  ) & 0xff;
    tmp[2] = ( ip >> 16 ) & 0xff;
    tmp[3] = ( ip >> 24 ) & 0xff;
#if defined(_MSC_VER) && (_MSC_VER >= 1400)
    sprintf_s( buffer, 256,
             "%3d.%3d.%3d.%3d", 
             tmp[0], tmp[1], tmp[2], tmp[3] );
#else
    sprintf( buffer,
             "%3d.%3d.%3d.%3d", 
             tmp[0], tmp[1], tmp[2], tmp[3] );
#endif
}

// ---- get proxy interfaceID ------------------------------------
static void
str_proxyID(
    ES::ChannelUtil::ProxyKind    kind,
    CORBA::Char                   **ppProxyID )
{
    switch( kind ) {
    case ES::ChannelUtil::PROXY_PULL_CONSUMER_EVENT:
        *ppProxyID = (CORBA::Char *)"IDL:CosEventChannelAdmin/ProxyPullConsumer:1.0";
        break;
    case ES::ChannelUtil::PROXY_PULL_SUPPLIER_EVENT:
        *ppProxyID = (CORBA::Char *)"IDL:CosEventChannelAdmin/ProxyPullSupplier:1.0";
        break;
    case ES::ChannelUtil::PROXY_PUSH_CONSUMER_EVENT:
        *ppProxyID = (CORBA::Char *)"IDL:CosEventChannelAdmin/ProxyPushConsumer:1.0";
        break;
    case ES::ChannelUtil::PROXY_PUSH_SUPPLIER_EVENT:
        *ppProxyID = (CORBA::Char *)"IDL:CosEventChannelAdmin/ProxyPushSupplier:1.0";
        break;
    case ES::ChannelUtil::PROXY_PULL_CONSUMER_NOTIFY:
        *ppProxyID = (CORBA::Char *)"IDL:CosNotifyChannelAdmin/ProxyPullConsumer:1.0";
        break;
    case ES::ChannelUtil::PROXY_PULL_SUPPLIER_NOTIFY:
        *ppProxyID = (CORBA::Char *)"IDL:CosNotifyChannelAdmin/ProxyPullSupplier:1.0";
        break;
    case ES::ChannelUtil::PROXY_PUSH_CONSUMER_NOTIFY:
        *ppProxyID = (CORBA::Char *)"IDL:CosNotifyChannelAdmin/ProxyPushConsumer:1.0";
        break;
    case ES::ChannelUtil::PROXY_PUSH_SUPPLIER_NOTIFY:
        *ppProxyID = (CORBA::Char *)"IDL:CosNotifyChannelAdmin/ProxyPushSupplier:1.0";
        break;
    case ES::ChannelUtil::STRUCTURED_PROXY_PULL_CONSUMER:
        *ppProxyID = (CORBA::Char *)"IDL:CosNotifyChannelAdmin/StructuredProxyPullConsumer:1.0";
        break;
    case ES::ChannelUtil::STRUCTURED_PROXY_PULL_SUPPLIER:
        *ppProxyID = (CORBA::Char *)"IDL:CosNotifyChannelAdmin/StructuredProxyPullSupplier:1.0";
        break;
    case ES::ChannelUtil::STRUCTURED_PROXY_PUSH_CONSUMER:
        *ppProxyID = (CORBA::Char *)"IDL:CosNotifyChannelAdmin/StructuredProxyPushConsumer:1.0";
        break;
    case ES::ChannelUtil::STRUCTURED_PROXY_PUSH_SUPPLIER:
        *ppProxyID = (CORBA::Char *)"IDL:CosNotifyChannelAdmin/StructuredProxyPushSupplier:1.0";
        break;
    default:
        *ppProxyID = (CORBA::Char *)UNKNOWN_PROXY;
        break;
    }
}

// ---- main function --------------------------------------------
int
main(
    int                                     argc,
    char                                    *argv[] )
{
    int                                     current_argc = argc;
    CORBA::ORB_ptr                          orb;
    CORBA::Object_ptr                       obj;
    CosNaming::NamingContext_ptr            naming_context;
    CosEventChannelAdmin::EventChannel_ptr  channel;
    ES::ChannelUtil_ptr                     util;
    CORBA::Environment                      env;
    const char                              *p;

    try {
        // ---- ORB initialize ----
        p = "ORB_init";
        orb = CORBA::ORB_init(
                        current_argc, argv, (CORBA::ORBid)FJ_OM_ORBid, env );

        p = "resolve_initial_references";
        obj = orb->resolve_initial_references(
                        (CORBA::ORB::ObjectId)CORBA_ORB_ObjectId_NameService, env );
        if ( obj == CosNaming::NamingContext::_nil() ) {
            printf( "CosNaming::NamingContext = NULL!\n" );
            return(1);
        }
        naming_context = CosNaming::NamingContext::_narrow( obj );
        CORBA::release( obj );

        // ---- EventChannel 'mix::sample' get ----
        CosNaming::Name_ptr             name;
        CosNaming::NameComponent_var    *name_component;

        name_component = CosNaming::Name::allocbuf( 1 );
        name_component[0]->id   = (const CORBA::Char *)"mix::sample";
        name_component[0]->kind = (const CORBA::Char *)"";
        name = new CosNaming::Name( 1, 1, name_component, CORBA_TRUE );
        p = "resolve";
        obj = naming_context->resolve( *name, env );
        delete  name;

        channel = CosEventChannelAdmin::EventChannel::_narrow( obj );
        CORBA::release( obj );

        // ---- create ES_Util interface object ----
        p = "create_util";
        obj= channel->create_util(
                    CosEventChannelAdmin::EventChannel::CHANNEL_UTIL, env );
        util = ES::ChannelUtil::_narrow( obj );
        CORBA::release( obj );

        // ---- get consumer count ----
        p = "get_proxys";
        ES::ChannelUtil::ProxyDataSeq
            *pProxyDataSeq = util->get_proxys( ES::ChannelUtil::ALL_PROXYS, env );

        // ---- print out ----
        unsigned long              i;
        char                       *pTime;
        char                       ip[256];
        CORBA::Char                *pProxyID;
        ES::ChannelUtil::ProxyData *pProxyData;

        printf( "----------------------------------------------------------\n" );
        printf( " No. IP address        Access time\n" );
        printf( "     Proxy kind \n" );
        printf( "----------------------------------------------------------\n" );
        for( i = 0; i < pProxyDataSeq->length(); i++ ) {
            pProxyData = (*pProxyDataSeq)[i];
            str_time( (time_t)pProxyData->time );
            str_ipaddress( pProxyData->ipaddress, ip );
            str_proxyID( pProxyData->kind, &pProxyID );
            printf( " %2d  %s   %s     %s\n", i+1, ip, TimeBuffer, pProxyID );
        }

    } catch( CORBA::SystemException   &se ) {
        printf( "Operation=%s, SystemException=%s, minor=%#x\n",
                p, se.id(), se.minor() );
        return(1);
    } catch( CORBA::Exception   &e ) {
        printf( "Operation=%s, Exception=%s\n", p, e.id() );
        return(1);
    }

    CORBA::release( util );
    CORBA::release( channel );
    CORBA::release( naming_context );

    return(0);
}

