// All Rights Reserved, Copyright (c) FUJITSU LIMITED 2002
#include    <stdlib.h>
#include    "orb_cplus.h"
#include    "NotificationService_cplus.h"
#ifdef OM_PC  /* Windows */
  #define EXIT ExitProcess
#else         /* Unix    */
  #define EXIT exit
#endif

// ---- create_channel -------------------------------------------
CosNotifyChannelAdmin::EventChannel_ptr
createChannel(
    CosNotifyChannelAdmin::EventChannelFactory_ptr    factory )
{
    CosNotification::QoSProperties                    *initial_qos;
    CosNotification::AdminProperties                  *initial_admin;
    CosNotifyChannelAdmin::ChannelID                  channelID;
    CosNotifyChannelAdmin::EventChannel_ptr           channel;
    CORBA::Environment                                env;
    CORBA::Char                                       *host;
    int                                               port;
    CORBA::Short                                      order;
    const char                                        *p;
    char                                              hostbuf[64+1];
    char                                              workbuf[64];

    try {
        // ---- set QoSProperties ----
        CosNotification::Property_var    *property;
        property = CosNotification::QoSProperties::allocbuf(1);
        if ( property == NULL ) {
            printf( "alloc error: new CosNotification::Property()\n" );
            EXIT(1);
        }
        // ---- OrderPolicy ----
        property[0]->name = CORBA::string_dup( CosNotification::OrderPolicy );
        if ( property[0]->name == NULL ) {
            printf( "alloc error: CORBA::string_dup(OrderPolicy)\n" );
            EXIT(1);
        }
        property[0]->value = new CORBA::Any();
        if ( property[0]->value == NULL ) {
            printf( "alloc error: new CORBA::Any()\n" );
            EXIT(1);
        }
        order = CosNotification::FifoOrder;
        *(property[0]->value) <<= order;
        
        initial_qos = new CosNotification::QoSProperties(
                              1, 1, property, CORBA_TRUE );
        if ( initial_qos == NULL ) {
            printf( "alloc error: new CosNotification::QosProperties()\n" );
            EXIT(1);
        }

        // ---- multi IP ----
        printf( "Do you use multi IP function?(y/n)" );
#if defined(_MSC_VER) && (_MSC_VER >= 1400)
        scanf_s( "%s", workbuf, sizeof(workbuf) );
#else
        scanf( "%s", workbuf );
#endif
        if (( workbuf[0] == 'y' )
         || ( workbuf[0] == 'Y' )) {
            // ---- get host ----
            printf( "Please input host name or IP address?" );
#if defined(_MSC_VER) && (_MSC_VER >= 1400)
            scanf_s( "%s", hostbuf, sizeof(hostbuf) );
#else
            scanf( "%s", hostbuf );
#endif

            // ---- get port ----
            printf( "Please input port number?" );
#if defined(_MSC_VER) && (_MSC_VER >= 1400)
            scanf_s( "%s", workbuf, sizeof(workbuf) );
            sscanf_s( workbuf, "%d", &port, sizeof(port) );
#else
            scanf( "%s", workbuf );
            sscanf( workbuf, "%d", &port );
#endif

            // ---- set AdminProperties ----
            property = CosNotification::AdminProperties::allocbuf(2);
            if ( property == NULL ) {
                printf( "alloc error: CosNotification::AdminProperties::allocbuf()\n" );
                EXIT(1);
            }
            // ---- HostName ----
            property[0]->name = CORBA::string_dup( "HostName" );
            if ( property[0]->name == NULL ) {
                printf( "alloc error: CORBA::string_dup(HostName)\n" );
                EXIT(1);
            }
            property[0]->value = new CORBA::Any();
            if ( property[0]->value == NULL ) {
                printf( "alloc error: new CORBA::Any()\n" );
                EXIT(1);
            }
            *(property[0]->value) <<= (CORBA::Char *)hostbuf;

            // ---- PortNumber ----
            property[1]->name = CORBA::string_dup( "PortNumber" );
            if ( property[1]->name == NULL ) {
                printf( "alloc error: CORBA::string_dup(PortNumber)\n" );
                EXIT(1);
            }
            property[1]->value = new CORBA::Any();
            if ( property[1]->value == NULL ) {
                printf( "alloc error: new CORBA::Any()\n" );
                EXIT(1);
            }
            *(property[1]->value) <<= (CORBA::UShort)port;

            initial_admin = new CosNotification::AdminProperties(
                                    2, 2, property, CORBA_TRUE );
            if ( initial_admin == NULL ) {
                printf( "alloc error: new CosNotification::AdminProperties()\n" );
                EXIT(1);
            }

        } else {
            initial_admin = new CosNotification::AdminProperties(
                                    0, 0, NULL, CORBA_FALSE );
            if ( initial_admin == NULL ) {
                printf( "alloc error: new CosNotification::AdminProperties()\n" );
                EXIT(1);
            }
        }

        // ---- create_channel ----
        p = "create_channel";
        channel = factory->create_channel( *initial_qos,
                                           *initial_admin,
                                           channelID,
                                           env );
        delete initial_admin;
        delete initial_qos;

    } catch( CORBA::SystemException &se ) {
        printf( "Operation=%s, SystemException=%s, minor=%#x\n",
                p, se.id(), se.minor() );
        EXIT(1);
    } catch( CORBA::Exception &e ) {
        printf( "Operation=%s, Exception=%s\n", p, e.id() );
        EXIT(1);
    }
    printf( "create EventChannelID = %d\n", channelID );

    return( channel );
}

// ---- get_all_channels -----------------------------------------
void
getAllChannels(
    CosNotifyChannelAdmin::EventChannelFactory_ptr    factory )
{
    CosNotifyChannelAdmin::ChannelIDSeq               *IDSeq;
    CORBA::Environment                                env;
    CosNotifyChannelAdmin::ChannelID                  ID;
    CORBA::ULong                                      count;
    const char                                        *p;

    // ---- get all channels ----
    try {
        p = "get_all_channels";
        IDSeq = factory->get_all_channels( env );
    } catch( CORBA::SystemException &se ) {
        printf( "Operation=%s, SystemException=%s, minor=%#x\n",
                p, se.id(), se.minor() );
        EXIT(1);
    } catch( CORBA::Exception &e ) {
        printf( "Operation=%s, Exception=%s\n", p, e.id() );
        EXIT(1);
    }

    printf( "--- EventChannelID list(ALL=%d) ---\n", (IDSeq->length()) );
    for( count=1; count<=(IDSeq->length()); count++ ) {
        ID = (*IDSeq)[count-1];
        printf( "%8d", ID );
        if ( count != (IDSeq->length()) ) {
            printf( "," );
        }
        if (( (count%8) == 0 )
         || ( count == (IDSeq->length()) )) {
            printf( "\n" );
        }
    }
    delete IDSeq;

    return;
}

// ---- get_event_channel ----------------------------------------
CosNotifyChannelAdmin::EventChannel_ptr
getEventChannel(
    CosNotifyChannelAdmin::EventChannelFactory_ptr    factory )
{
    CosNotifyChannelAdmin::ChannelID                  channelID;
    CosNotifyChannelAdmin::EventChannel_ptr           channel;
    CORBA::Environment                                env;
    const char                                        *p;
    char                                              workbuf[64];

    channelID = 0;
    printf( "input EventChannelID:" );
#if defined(_MSC_VER) && (_MSC_VER >= 1400)
    scanf_s( "%s", workbuf, sizeof(workbuf) );
    sscanf_s( workbuf, "%d", &channelID, sizeof(channelID) );
#else
    scanf( "%s", workbuf );
    sscanf( workbuf, "%d", &channelID );
#endif

    // ---- get event channel ----
    try {
        p = "get_event_channel";
        channel = factory->get_event_channel( channelID,
                                              env );
    } catch( CosNotifyChannelAdmin::ChannelNotFound &e ) {
        printf( "EventChannel[ID=%d] Exception=%s\n",
                channelID, e.id() );
        return( CosNotifyChannelAdmin::EventChannel::_nil() );
    } catch( CORBA::SystemException &se ) {
        printf( "Operation=%s, SystemException=%s, minor=%#x\n",
                p, se.id(), se.minor() );
        EXIT(1);
    } catch( CORBA::Exception &e ) {
        printf( "Operation=%s, Exception=%s\n", p, e.id() );
        EXIT(1);
    }
    printf( "get EventChannel Object\n" );

    return( channel );
}

// ---- main function --------------------------------------------
int
main(
    int         argc,
    char        *argv[] )
{
    int                         current_argc = argc;
    CORBA::ORB_ptr              orb;
    CORBA::Environment          env;
    CORBA::Object_ptr           obj;
    const char                  *p;
    char                        workbuf[64];

    try {
        // ---- ORB initialize ----
        p = "ORB_init";
        orb = CORBA::ORB_init(
                        current_argc, argv, (CORBA::ORBid)FJ_OM_ORBid, env );

        // ---- get NotificationService Factory ----
        p = "resolve_initial_references";
        obj = orb->resolve_initial_references(
                        (CORBA::ORB::ObjectId)"NotificationService", env );
        if ( obj == CosNotifyChannelAdmin::EventChannelFactory::_nil() ) {
            printf( "Factory is not setup.\n" );
            return(1);
        }
        CosNotifyChannelAdmin::EventChannelFactory_ptr   factory;
        factory = CosNotifyChannelAdmin::EventChannelFactory::_narrow( obj );
        CORBA::release( obj );

        // ---- select EventChannel ----
        CosNotifyChannelAdmin::EventChannel_ptr    channel
                = CosNotifyChannelAdmin::EventChannel::_nil();
        int    select;
        do {
            printf( "1.create_channel, 2.get_all_channels, 3.get_event_channel\n" );
            printf( "select(1-3):" );
#if defined(_MSC_VER) && (_MSC_VER >= 1400)
            scanf_s( "%s", workbuf, sizeof(workbuf) );
            sscanf_s( workbuf, "%d", &select, sizeof(select) );
#else
            scanf( "%s", workbuf );
            sscanf( workbuf, "%d", &select );
#endif
            switch( select ) {
            case 1:
                channel = createChannel( factory );
                break;
            case 2:
                getAllChannels( factory );
                break;
            case 3:
                channel = getEventChannel( factory );
                break;
            default:
                printf( "parameter error\n\n" );
            }
        } while( channel == CosNotifyChannelAdmin::EventChannel::_nil() );

        printf( "connecting...\n" );

        // ---- connect EventChannel ----
        CosNotifyChannelAdmin::ConsumerAdmin_ptr         admin;
        CosNotifyChannelAdmin::ProxySupplier_ptr         tmp_proxy;
        CosNotifyChannelAdmin::ProxyPullSupplier_ptr     proxy;
        CosNotifyChannelAdmin::ProxyID                   proxyID;

        p = "default_consumer_admin";
        admin = channel->default_consumer_admin( env );
        p = "obtain_notification_pull_supplier";
        tmp_proxy = admin->obtain_notification_pull_supplier(
                        CosNotifyChannelAdmin::ANY_EVENT, proxyID, env );
        proxy = CosNotifyChannelAdmin::ProxyPullSupplier::_narrow( tmp_proxy );
        CORBA::release( tmp_proxy );
        p = "connect_any_pull_consumer";
        proxy->connect_any_pull_consumer(
                        CosEventComm::PullConsumer::_nil(), env );

        CORBA::Any    *recv;
        try {
            p = "pull";
            recv = proxy->pull( env );
        } catch( CORBA::SystemException &se ) {
            printf( "Operation=%s, SystemException=%s, minor=%#x\n",
                    p, se.id(), se.minor() );
             try {
                 proxy->disconnect_pull_supplier( env );
             } catch( CORBA::Exception &e ) {}
            return(1);
        } catch( CORBA::Exception &e ) {
            printf( "Operation=%s, Exception=%s\n", p, e.id() );
             try {
                 proxy->disconnect_pull_supplier( env );
             } catch( CORBA::Exception &e ) {}
            return(1);
        }

        CORBA::Char *message = (CORBA::Char *)recv->value();
        printf( "pull message : %s\n", message );
        delete recv;

        p = "disconnect_pull_supplier";
        proxy->disconnect_pull_supplier( env );

        CORBA::release( proxy );
        CORBA::release( admin );
        CORBA::release( channel );
        CORBA::release( factory );

    } catch( CORBA::SystemException &se ) {
        printf( "Operation=%s, SystemException=%s, minor=%#x\n",
                p, se.id(), se.minor() );
        return(1);
    } catch( CORBA::Exception &e ) {
        printf( "Operation=%s, Exception=%s\n", p, e.id() );
        return(1);
    }

    return(0);
}

