// All Rights Reserved, Copyright (c) FUJITSU LIMITED 2002
#include    <stdlib.h>
#include    <string.h>
#include    "orb_cplus.h"
#include    "CosNaming_cplus.h"
#include    "NotificationService_cplus.h"

// ---- create StructuredEvent --------------------------
CosNotification::StructuredEvent *
makeData()
{
    CosNotification::StructuredEvent    *event =
        new CosNotification::StructuredEvent();

    CORBA::Short        priority = 1;
    CORBA::ULongLong    timeout  = 40;

    CosNotification::Property_var    *pProperty =
        CosNotification::PropertySeq::allocbuf( 2 );

    // ---- Priority ------------------------------------
    pProperty[0]->name  = CORBA::string_dup( CosNotification::Priority );
    pProperty[0]->value = new CORBA::Any();
    *(pProperty[0]->value) <<= priority;

    // ---- Timeout -------------------------------------
    pProperty[1]->name  = CORBA::string_dup( CosNotification::Timeout );
    pProperty[1]->value = new CORBA::Any();
    *(pProperty[1]->value) <<= timeout;

    event->header->variable_header =
        new CosNotification::OptionalHeaderFields(
                                2, 2, pProperty, CORBA_TRUE );

    // ---- EventHeader ---------------------------------
    event->header->fixed_header->event_type->domain_name =
        CORBA::string_dup( "Telecom" );

    event->header->fixed_header->event_type->type_name =
        CORBA::string_dup( "CommunicationsAlarm" );

    event->header->fixed_header->event_name =
        CORBA::string_dup( "data" );

    // ---- EventBody -----------------------------------
    event->filterable_data = 
        new CosNotification::FilterableEventBody(
                                0, 0, NULL, CORBA_FALSE );

    CORBA::Char *string_data = CORBA::string_dup( "StructuredEvent Mixed-C++" );
    event->remainder_of_body =
        new CORBA::Any( _tc_string, string_data, CORBA_TRUE );

    return( event );
}

// ---- main function --------------------------------------------
int
main(
    int                   argc,
    char                  *argv[] )
{
    int                   current_argc = argc;
    CORBA::ORB_ptr        orb;
    CORBA::Environment    env;
    CORBA::Object_ptr     obj;
    const char            *p;

    try {
        // ---- ORB initialize ----
        p = "ORB_init";
        orb = CORBA::ORB_init(
                        current_argc, argv, (CORBA::ORBid)FJ_OM_ORBid, env );

        p = "resolve_initial_references";
        obj = orb->resolve_initial_references(
                        (CORBA::ORB::ObjectId)CORBA_ORB_ObjectId_NameService, env );
        if ( obj == CosNaming::NamingContext::_nil() ) {
            printf( "CosNaming::NamingContext = NULL!\n" );
            return(1);
        }

        CosNaming::NamingContext_ptr    naming_context;
        naming_context = CosNaming::NamingContext::_narrow( obj );
        CORBA::release( obj );

        // ---- EventChannel 'mix::sample' get ----
        CosNaming::Name_ptr             name;
        CosNaming::NameComponent_var    *name_component;

        name_component = CosNaming::Name::allocbuf( 1 );
        name_component[0]->id   = (const CORBA::Char *)"mix::sample";
        name_component[0]->kind = (const CORBA::Char *)"";
        name = new CosNaming::Name( 1, 1, name_component, CORBA_TRUE );
        p = "resolve";
        obj = naming_context->resolve( *name, env );
        delete name;

        CosNotifyChannelAdmin::EventChannel_ptr    channel;
        channel = CosNotifyChannelAdmin::EventChannel::_narrow( obj );
        CORBA::release( obj );

        // ---- connect EventChannel ----
        CosNotifyChannelAdmin::SupplierAdmin_ptr         admin;
        CosNotifyChannelAdmin::ProxyConsumer_ptr         tmp_proxy;
        CosNotifyChannelAdmin::StructuredProxyPushConsumer_ptr     proxy;
        CosNotifyChannelAdmin::ProxyID                   proxyID;

        p = "default_supplier_admin";
        admin = channel->default_supplier_admin( env );
        p = "obtain_notification_push_consumer";
        tmp_proxy = admin->obtain_notification_push_consumer(
                        CosNotifyChannelAdmin::STRUCTURED_EVENT, proxyID, env );
        proxy = CosNotifyChannelAdmin::StructuredProxyPushConsumer::_narrow( tmp_proxy );
        CORBA::release( tmp_proxy );
        p = "connect_structured_push_supplier";
        proxy->connect_structured_push_supplier(
                        CosNotifyComm::StructuredPushSupplier::_nil(), env );

        CosNotification::StructuredEvent *send = makeData();

        try {
            p = "push_structured_event";
            proxy->push_structured_event( *send, env );
        } catch( CORBA::SystemException &se ) {
            printf( "Operation=%s, SystemException=%s, minor=%#x\n",
                    p, se.id(), se.minor() );
            try {
                proxy->disconnect_structured_push_consumer( env );
            } catch( CORBA::Exception e ) {}
            return(1);
        } catch( CORBA::Exception &e ) {
            printf( "Operation=%s, Exception=%s\n", p, e.id() );
            try {
                proxy->disconnect_structured_push_consumer( env );
            } catch( CORBA::Exception e ) {}
            return(1);
        }
        delete send;

        p = "disconnect_structured_push_consumer";
        proxy->disconnect_structured_push_consumer( env );

        CORBA::release( proxy );
        CORBA::release( admin );
        CORBA::release( channel );

    } catch( CORBA::SystemException   &se ) {
        printf( "Operation=%s, SystemException=%s, minor=%#x\n",
                p, se.id(), se.minor() );
        return(1);
    } catch( CORBA::Exception   &e ) {
        printf( "Operation=%s, Exception=%s\n", p, e.id() );
        return(1);
    }

    return(0);
}

