// All Rights Reserved, Copyright (c) FUJITSU LIMITED 2002
#include    <stdlib.h>
#include    <string.h>
#include    "orb_cplus.h"
#include    "EventService_cplus.h"
#ifdef OM_PC  /* Windows */
  #define EXIT ExitProcess
#else         /* Unix    */
  #define EXIT exit
#endif

// ---- create ---------------------------------------------------
CosEventChannelAdmin::EventChannel_ptr
create(
    EventFactory_ptr    factory,
    CORBA::Char         *key )
{
    CosEventChannelAdmin::EventChannel_ptr    channel;
    CORBA::Environment  env;
    const char          *p;

    try {
        // ---- create EventChannel ----
        EventFactory::Option                    opt;
        opt.max_queuing = EventFactory::ES_DEFAULT_VALUE;
        opt.life_time   = EventFactory::ES_DEFAULT_VALUE;
        opt.model       = EventFactory::ModelMixed;

        p = "create";
        channel = factory->create( key, opt, env );

    } catch( CORBA::SystemException &se ) {
        printf( "Operation=%s, SystemException=%s, minor=%#x\n",
                p, se.id(), se.minor() );
        EXIT(1);
    } catch( CORBA::Exception &e ) {
        printf( "Operation=%s, Exception=%s\n", p, e.id() );
        EXIT(1);
    }
    printf( "create EventChannel(%s)\n", key );

    return( channel );
}


// ---- create_channel -------------------------------------------
CosEventChannelAdmin::EventChannel_ptr
createChannel(
    EventFactory_ptr    factory,
    CORBA::Char         *key )
{
    CosEventChannelAdmin::EventChannel_ptr    channel;
    CORBA::Environment                        env;
    EventFactory::EventProperty               *event_property;
    EventFactory::Property_var                *property;
    const char                                *p;
    char                                      workbuf[64];
    char                                      hostbuf[64+1];
    int                                       port;
    CORBA::Boolean                            create;

    try {
        // ---- create EventChannel ----
        printf( "Do you use multi IP function(y/n)" );
#if defined(_MSC_VER) && (_MSC_VER >= 1400)
        scanf_s( "%s", workbuf, sizeof(workbuf) );
#else
        scanf( "%s", workbuf );
#endif
        if (( workbuf[0] == 'y' )
         || ( workbuf[0] == 'Y' )) {
            // ---- get host ----
            printf( "Please input host name or IP address?" );
#if defined(_MSC_VER) && (_MSC_VER >= 1400)
            scanf_s( "%s", hostbuf, sizeof(hostbuf) );
#else
            scanf( "%s", hostbuf );
#endif

            // ---- get port ----
            printf( "Please input port number?" );
#if defined(_MSC_VER) && (_MSC_VER >= 1400)
            scanf_s( "%s", workbuf, sizeof(workbuf) );
            sscanf_s( workbuf, "%d", &port, sizeof(port) );
#else
            scanf( "%s", workbuf );
            sscanf( workbuf, "%d", &port );
#endif

            // ---- set EventProperty ----
            property = EventFactory::EventProperty::allocbuf(2);
            if ( property == NULL ) {
                printf( "alloc error: EventFactory::EventProperty::allocbuf()\n" );
                EXIT(1);
            }

            // ---- HostName ----
            property[0]->name = CORBA::string_dup( "HostName" );
            if ( property[0]->name == NULL ) {
                printf( "alloc error: CORBA::string_dup(HostName)\n" );
                EXIT(1);
            }
            property[0]->value = new CORBA::Any();
            if ( property[0]->value == NULL ) {
                printf( "alloc error: new CORBA::Any()\n" );
                EXIT(1);
            }
            *(property[0]->value) <<= (CORBA::Char *)hostbuf;

            // ---- PortNumber ----
            property[1]->name = CORBA::string_dup( "PortNumber" );
            if ( property[1]->name == NULL ) {
                printf( "alloc error: CORBA::string_dup(PortNumber)\n" );
                EXIT(1);
            }
            property[1]->value = new CORBA::Any();
            if ( property[1]->value == NULL ) {
                printf( "alloc error: new CORBA::Any()\n" );
                EXIT(1);
            }
            *(property[1]->value) <<= (CORBA::UShort)port;

            event_property = new EventFactory::EventProperty(
                                     2, 2, property, CORBA_TRUE );
            if ( event_property == NULL ) {
                printf( "alloc error: new EventFactory::EventProperty()\n" );
                EXIT(1);
            }

        } else {
            event_property = new EventFactory::EventProperty(
                                     0, 0, NULL, CORBA_FALSE );
            if ( event_property == NULL ) {
                printf( "alloc error: new EventFactory::EventProperty()\n" );
                EXIT(1);
            }
        }

        // ---- set option parameter ----
        EventFactory::Option                    opt;
        opt.max_queuing = EventFactory::ES_DEFAULT_VALUE;
        opt.life_time   = EventFactory::ES_DEFAULT_VALUE;
        opt.model       = EventFactory::ModelMixed;

        // ---- create_channel ----
        p = "create_channel";
        channel = factory->create_channel( key,
                                           opt,
                                           *event_property,
                                           create,
                                           env );
        delete event_property;

    } catch( CORBA::SystemException &se ) {
        printf( "Operation=%s, SystemException=%s, minor=%#x\n",
                p, se.id(), se.minor() );
        EXIT(1);
    } catch( CORBA::Exception &e ) {
        printf( "Operation=%s, Exception=%s\n", p, e.id() );
        EXIT(1);
    }

    if ( create == CORBA_TRUE ) {
        printf( "create EventChannel(%s) Object\n", key );
    } else {
        printf( "get EventChannel(%s) Object\n", key );
    }

    return( channel );
}


// ---- get_event_channel ----------------------------------------
CosEventChannelAdmin::EventChannel_ptr
getEventChannel(
    EventFactory_ptr    factory,
    CORBA::Char         *key )
{
    CosEventChannelAdmin::EventChannel_ptr    channel;
    CORBA::Environment  env;
    const char          *p;

    try {
        // ---- get event channel ----
        p = "get_event_channel";
        channel = factory->get_event_channel( key, env );

    } catch( CORBA::SystemException &se ) {
        printf( "Operation=%s, SystemException=%s, minor=%#x\n",
                p, se.id(), se.minor() );
        EXIT(1);
    } catch( CORBA::Exception &e ) {
        printf( "Operation=%s, Exception=%s\n", p, e.id() );
        EXIT(1);
    }
    printf( "get EventChannel(%s) Object\n", key );

    return( channel );
}


// ---- main function --------------------------------------------
int
main(
    int                 argc,
    char                *argv[] )
{
    int                 current_argc = argc;
    CORBA::ORB_ptr      orb;
    CORBA::Environment  env;
    const char          *p;
    char                workbuf[64];
    char                keybuf[64+1];

    try {
        // ---- ORB initialize ----
        p = "ORB_init";
        orb = CORBA::ORB_init(
                        current_argc, argv, (CORBA::ORBid)FJ_OM_ORBid, env );

        // ---- get EventFactory object ----
        CORBA::Object_ptr       obj;
        EventFactory_ptr        factory;
        p = "resolve_initial_references";
        obj = orb->resolve_initial_references(
                        (CORBA::ORB::ObjectId)EventFactory::ObjectId_Factory, env );
        if ( obj == EventFactory::_nil() ) {
            printf( "Factory is not setup.\n" );
            return(1);
        }
        factory = EventFactory::_narrow( obj );
        CORBA::release( obj );

        // ---- select EventChannel ----
        CosEventChannelAdmin::EventChannel_ptr    channel
            = CosEventChannelAdmin::EventChannel::_nil();
        int    select;
        do {
            printf( "1.create, 2.create_channel, 3.get_event_channel\n" );
            printf( "select(1-3):" );
#if defined(_MSC_VER) && (_MSC_VER >= 1400)
            scanf_s( "%s", workbuf, sizeof(workbuf) );
            sscanf_s( workbuf, "%d", &select, sizeof(select) );
#else
            scanf( "%s", workbuf );
            sscanf( workbuf, "%d", &select );
#endif
            printf( "input EventChannel name:" );
#if defined(_MSC_VER) && (_MSC_VER >= 1400)
            scanf_s( "%s", keybuf, sizeof(keybuf) );
#else
            scanf( "%s", keybuf );
#endif
            switch( select ) {
            case 1:
                channel = create( factory, keybuf );
                break;
            case 2:
                channel = createChannel( factory, keybuf );
                break;
            case 3:
                channel = getEventChannel( factory, keybuf );
                break;
            default:
                printf( "parameter error\n\n" );
            }
        } while( channel == CORBA_OBJECT_NIL );

        // ---- connect EventChannel ----
        CosEventChannelAdmin::SupplierAdmin_ptr    admin;
        p = "for_suppliers";
        admin = channel->for_suppliers( env );

        CosEventChannelAdmin::ProxyPushConsumer_ptr    proxy;
        p = "obtain_push_consumer";
        proxy = admin->obtain_push_consumer( env );
        p = "connect_push_supplier";
        proxy->connect_push_supplier(
                        CosEventComm::PushSupplier::_nil(), env );

        // ---- push ----
        CORBA::Any      *send;
        CORBA::Char     *message;
        send = new CORBA::Any();
        if ( send == NULL ) {
            printf( "alloc error: new CORBA::Any()\n" );
            return(1);
        }
        message = CORBA::string_alloc( strlen("Factory-MIX-C++") );
        if ( message == NULL ) {
            printf( "alloc error: CORBA::string_alloc()\n" );
            return(1);
        }
#if defined(_MSC_VER) && (_MSC_VER >= 1400)
        strcpy_s( message, strlen("Factory-MIX-C++")+1, "Factory-MIX-C++" );
#else
        strcpy( message, "Factory-MIX-C++" );
#endif
        send->replace( _tc_string, message, CORBA_TRUE );

        try {
            p = "push";
            proxy->push( *send, env );
        } catch( CORBA::SystemException &se ) {
            printf( "Operation=%s, SystemException=%s, minor=%#x\n",
                    p, se.id(), se.minor() );
            try {
                proxy->disconnect_push_consumer( env );
            } catch( CORBA::Exception &e ) {}
            delete send;
            return(1);
        } catch( CORBA::Exception &e ) {
            printf( "Operation=%s, Exception=%s\n", p, e.id() );
            try {
                proxy->disconnect_push_consumer( env );
            } catch( CORBA::Exception &e ) {}
            delete send;
            return(1);
        }
        delete send;

        // ---- disconnect EventChannel ----
        p = "disconnect_push_consumer";
        proxy->disconnect_push_consumer( env );

        CORBA::release( proxy );
        CORBA::release( admin );
        CORBA::release( channel );

    } catch( CORBA::SystemException &se ) {
        printf( "Operation=%s, SystemException=%s, minor=%#x\n",
                p, se.id(), se.minor() );
        return(1);
    } catch( CORBA::Exception &e ) {
        printf( "Operation=%s, Exception=%s\n", p, e.id() );
        return(1);
    }

    return(0);
}

