// All Rights Reserved, Copyright (c) FUJITSU LIMITED 2002
#include    <stdlib.h>
#include    <string.h>
#include    "orb_cplus.h"
#include    "CosNaming_cplus.h"
#include    "EventService_cplus.h"

// ---- main function --------------------------------------------
int
main(
    int                         argc,
    char                        *argv[] )
{
    int                         current_argc = argc;
    CORBA::ORB_ptr              orb;
    CORBA::BOA_ptr              boa;
    CORBA::Environment          env;
    CORBA::Repository_ptr       intf_rep;
    FJ::ImplementationRep_ptr   impl_rep;
    CORBA::Object_ptr           obj;
    const char                  *p;

    try {
        // ---- ORB initialize ----
        p = "ORB_init";
        orb = CORBA::ORB_init(
                        current_argc, argv, (CORBA::ORBid)FJ_OM_ORBid, env );

        p = "BOA_init";
        boa = orb->BOA_init(
                        current_argc, argv, (CORBA::ORB::OAid)CORBA_BOA_OAid, env );

        p = "resolve_initial_references(IntfRep)";
        obj = orb->resolve_initial_references(
                        (CORBA::ORB::ObjectId)CORBA_ORB_ObjectId_LightInterfaceRepository, env );
        if ( obj == CORBA::Repository::_nil() ) {
            printf( "CORBA::Repository = NULL!\n" );
            return(1);
        }
        intf_rep = CORBA::Repository::_narrow( obj );
        CORBA::release( obj );

        p = "resolve_initial_references(ImplRep)";
        obj = orb->resolve_initial_references(
                        (CORBA::ORB::ObjectId)CORBA_ORB_ObjectId_ImplementationRepository, env );
        if ( obj == FJ::ImplementationRep::_nil() ) {
            printf( "FJ::ImplementationRep = NULL!\n" );
            return(1);
        }
        impl_rep = FJ::ImplementationRep::_narrow( obj );
        CORBA::release( obj );

        p = "resolve_initial_references(Name)";
        obj = orb->resolve_initial_references(
                        (CORBA::ORB::ObjectId)CORBA_ORB_ObjectId_NameService, env );
        if ( obj == CosNaming::NamingContext::_nil() ) {
            printf( "CosNaming::NamingContext = NULL!\n" );
            return(1);
        }
        CosNaming::NamingContext_ptr    naming_context;
        naming_context = CosNaming::NamingContext::_narrow( obj );
        CORBA::release( obj );

        // ---- EventChannel 'push::sample' get ----
        CosNaming::Name_ptr             name;
        CosNaming::NameComponent_var    *name_component;

        name_component = CosNaming::Name::allocbuf( 1 );
        name_component[0]->id   = (const CORBA::Char *)"push::sample";
        name_component[0]->kind = (const CORBA::Char *)"";
        name = new CosNaming::Name( 1, 1, name_component, CORBA_TRUE );
        p = "resolve";
        obj = naming_context->resolve( *name, env );
        delete name;

        CosEventChannelAdmin::EventChannel_ptr    channel;
        channel = CosEventChannelAdmin::EventChannel::_narrow( obj );
        CORBA::release( obj );

        // ---- create MyObject reference ----
        CORBA::InterfaceDef_ptr    intf;
        p = "lookup_id(Intf)";
        obj = intf_rep->lookup_id( _INTF_CosEventComm_PushConsumer, env );
        intf = CORBA::InterfaceDef::_narrow( obj );
        CORBA::release( obj );

        CORBA::ImplementationDef_ptr    impl;
        p = "lookup_id(Impl)";
        obj = impl_rep->lookup_id( (FJ::RepositoryId)"IDL:push/sampleCPP:1.0", env );
        impl = CORBA::ImplementationDef::_narrow( obj );
        CORBA::release( obj );

        CosEventComm::PushConsumer_ptr    my_object;
        CORBA::ReferenceData              refid;
        p = "create";
        obj = boa->create( refid, intf, impl, env );
        my_object = CosEventComm::PushConsumer::_narrow( obj );
        CORBA::release( obj );

        // ---- connect EventChannel ----
        CosEventChannelAdmin::ConsumerAdmin_ptr         admin;
        CosEventChannelAdmin::ProxyPushSupplier_ptr     proxy;

        p = "for_consumers";
        admin = channel->for_consumers( env );
        p = "obtain_push_supplier";
        proxy = admin->obtain_push_supplier( env );
        p = "connect_push_consumer";
        proxy->connect_push_consumer( my_object, env );

        try {
            p = "impl_is_ready";
            boa->impl_is_ready( impl, env );
        } catch( CORBA::SystemException &se ) {
            printf( "Operation=%s, SystemException=%s, minor=%#x\n",
                    p, se.id(), se.minor() );
            try {
                proxy->disconnect_push_supplier( env );
            } catch( CORBA::Exception &e ) {}
            return(1);
        } catch( CORBA::Exception &e ) {
            printf( "Operation=%s, Exception=%s\n", p, e.id() );
            try {
                proxy->disconnect_push_supplier( env );
            } catch( CORBA::Exception &e ) {}
            return(1);
        }

        printf( "Push [Enter/Return] Key\n" );
        getchar();

        p = "disconnect_push_supplier";
        proxy->disconnect_push_supplier( env );

        CORBA::release( proxy );
        CORBA::release( admin );
        CORBA::release( channel );

    } catch( CORBA::SystemException   &se ) {
        printf( "Operation=%s, SystemException=%s, minor=%#x\n",
                p, se.id(), se.minor() );
        return(1);
    } catch( CORBA::Exception   &e ) {
        printf( "Operation=%s, Exception=%s\n", p, e.id() );
        return(1);
    }

    return(0);
}

