// All Rights Reserved, Copyright (c) FUJITSU LIMITED 2002
#include    <stdio.h>
#include    <stdlib.h>
#include    "orb_cplus.h"
#include    "CosNaming_cplus.h"
#include    "EventService_cplus.h"

// ---- main function --------------------------------------------
int
main(
    int                                     argc,
    char                                    *argv[] )
{
    int                                     current_argc = argc;
    CORBA::ORB_ptr                          orb;
    CORBA::Object_ptr                       obj;
    CosNaming::NamingContext_ptr            naming_context;
    CosEventChannelAdmin::EventChannel_ptr  channel;
    ES::ChannelUtil_ptr                     util;
    CORBA::Environment                      env;
    const char                              *p;

    try {
        // ---- ORB initialize ----
        p = "ORB_init";
        orb = CORBA::ORB_init(
                        current_argc, argv, (CORBA::ORBid)FJ_OM_ORBid, env );

        p = "resolve_initial_references";
        obj = orb->resolve_initial_references(
                        (CORBA::ORB::ObjectId)CORBA_ORB_ObjectId_NameService, env );
        if ( obj == CosNaming::NamingContext::_nil() ) {
            printf( "CosNaming::NamingContext = NULL!\n" );
            return(1);
        }
        naming_context = CosNaming::NamingContext::_narrow( obj );
        CORBA::release( obj );

        // ---- EventChannel 'mix::sample' get ----
        CosNaming::Name_ptr             name;
        CosNaming::NameComponent_var    *name_component;

        name_component = CosNaming::Name::allocbuf( 1 );
        name_component[0]->id   = (const CORBA::Char *)"mix::sample";
        name_component[0]->kind = (const CORBA::Char *)"";
        name = new CosNaming::Name( 1, 1, name_component, CORBA_TRUE );
        p = "resolve";
        obj = naming_context->resolve( *name, env );
        delete  name;

        channel = CosEventChannelAdmin::EventChannel::_narrow( obj );
        CORBA::release( obj );

        // ---- create ES_Util interface object ----
        p = "create_util";
        obj= channel->create_util(
                    CosEventChannelAdmin::EventChannel::CHANNEL_UTIL, env );
        util = ES::ChannelUtil::_narrow( obj );
        CORBA::release( obj );

        // ---- get consumer count ----
        p = "get_consumer_count";
        CORBA::ULong consumer = util->get_consumer_count( env );

        // ---- get supplier count ----
        p = "get_supplier_count";
        CORBA::ULong supplier = util->get_supplier_count( env );

        // ---- get queue length ----
        p = "get_queue_length";
        CORBA::ULong queue = util->get_queue_length( env );

        // ---- print out ----
        printf( " Consumer count = %d\n", consumer );
        printf( " Supplier count = %d\n", supplier );
        printf( " Queue length   = %d\n", queue );

    } catch( CORBA::SystemException   &se ) {
        printf( "Operation=%s, SystemException=%s, minor=%#x\n",
                p, se.id(), se.minor() );
        return(1);
    } catch( CORBA::Exception   &e ) {
        printf( "Operation=%s, Exception=%s\n", p, e.id() );
        return(1);
    }

    CORBA::release( util );
    CORBA::release( channel );
    CORBA::release( naming_context );

    return(0);
}

