// All Rights Reserved, Copyright (c) FUJITSU LIMITED 2002
#include    <stdlib.h>
#include    <string.h>
#include    <errno.h>

#include    "orb_cplus.h"
#include    "CosNaming_cplus.h"
#include    "NotificationService_cplus.h"

// ---- usage function ------------------------------------------
static void
usage()
{
    printf( "parameter error\n" );
    printf( "Usage:mix_consumer ID [-del]\n" );
    printf( "ID: proxy ID\n" );
    printf( "-del: proxy delete mode\n" );
}

// ---- main function --------------------------------------------
int
main(
    int                 argc,
    char                *argv[] )
{
    int                 current_argc = argc;
    CORBA::ORB_ptr      orb;
    CORBA::Environment  env;
    CORBA::Object_ptr   obj;
    const char          *p;
    ES::ChannelUtil_ptr util;
    FILE                *fd;
    CORBA::Char         *str;
    CORBA::Boolean      del_proxy = CORBA_FALSE;
    CORBA::Char         filename[64];
    int                 count;
#if defined(_MSC_VER) && (_MSC_VER >= 1400)
    errno_t             err;
#endif

    memset( filename, '\0', sizeof(filename) );
    for( count=1; count<argc; count++ ) {
        if ( strcmp( argv[count], "-del" ) == 0 ) {
            if ( del_proxy == CORBA_FALSE ) {
                del_proxy = CORBA_TRUE;
                continue;
            } else {
                usage();
                return(1);
            }
        } else
        if ( filename[0] != '\0' ) {
            usage();
            return(1);
        }
#if defined(_MSC_VER) && (_MSC_VER >= 1400)
        sprintf_s( filename, sizeof(filename), "proxy%.58s", argv[count] );
#else
        sprintf( filename, "proxy%.58s", argv[count] );
#endif
    } /* for */
    if ( filename[0] == '\0' ) {
        usage();
        return(1);
    }
    if ( del_proxy == CORBA_TRUE ) {
#if defined(_MSC_VER) && (_MSC_VER >= 1400)
        err = fopen_s( &fd, filename, "r" );
        if ( err != 0 ) {
            printf( "This ID is not entry.\n" );
            return(1);
        }
#else
        fd = fopen( filename, "r" );
        if ( fd == NULL ) {
            printf( "This ID is not entry.\n" );
            return(1);
        }
#endif
        fclose(fd);
    }

    try {
        CosNotifyChannelAdmin::ConsumerAdmin_ptr                  admin;
        CosNotifyChannelAdmin::ProxySupplier_ptr                  tmp_proxy;
        CosNotifyChannelAdmin::StructuredProxyPullSupplier_ptr    proxy;
        CosNotifyChannelAdmin::ProxyID                            proxyID;

        // ---- ORB initialize ----
        p = "ORB_init";
        orb = CORBA::ORB_init(
                        current_argc, argv, (CORBA::ORBid)FJ_OM_ORBid, env );

        p = "resolve_initial_references";
        obj = orb->resolve_initial_references(
                        (CORBA::ORB::ObjectId)CORBA_ORB_ObjectId_NameService, env );
        if ( obj == CosNaming::NamingContext::_nil() ) {
            printf( "CosNaming::NamingContext = NULL!\n" );
            return(1);
        }

        CosNaming::NamingContext_ptr    naming_context;
        naming_context = CosNaming::NamingContext::_narrow( obj );
        CORBA::release( obj );

        // ---- EventChannel 'mix::sample' get ----
        CosNaming::Name_ptr             name;
        CosNaming::NameComponent_var    *name_component;

        name_component = CosNaming::Name::allocbuf( 1 );
        name_component[0]->id   = (const CORBA::Char *)"mix::sample";
        name_component[0]->kind = (const CORBA::Char *)"";
        name = new CosNaming::Name( 1, 1, name_component, CORBA_TRUE );
        p = "resolve";
        obj = naming_context->resolve( *name, env );
        delete  name;

        CosNotifyChannelAdmin::EventChannel_ptr    channel;
        channel = CosNotifyChannelAdmin::EventChannel::_narrow( obj );
        CORBA::release( obj );

#if defined(_MSC_VER) && (_MSC_VER >= 1400)
        err = fopen_s( &fd, filename, "r" );
        if ( err == 0 ) {
#else
        fd = fopen( filename, "r" );
        if ( fd != NULL ) {
#endif
            str = CORBA::string_alloc(4096);
#if defined(_MSC_VER) && (_MSC_VER >= 1400)
            fscanf_s( fd, "%s", str, (4096+1) );
#else
            fscanf( fd, "%s", str );
#endif
            fclose( fd );
            obj = orb->string_to_object( str, env );
            proxy = CosNotifyChannelAdmin::StructuredProxyPullSupplier::_narrow( obj );
            CORBA::string_free(str);
            CORBA::release( obj );
            if ( del_proxy == CORBA_TRUE ) {
                if ( remove( filename ) == -1 ) {
                    printf( "%s remove error! %d\n", filename, errno );
                    return(1);
                }

                proxy->disconnect_structured_pull_supplier( env );
                CORBA::release( proxy );
                return(0);
            }
        } else {
            // ---- connect EventChannel ----
            p = "default_consumer_admin";
            admin = channel->default_consumer_admin( env );
            p = "obtain_notification_pull_supplier";
            tmp_proxy = admin->obtain_notification_pull_supplier(
                            CosNotifyChannelAdmin::STRUCTURED_EVENT, proxyID, env );
            proxy = CosNotifyChannelAdmin::StructuredProxyPullSupplier::_narrow( tmp_proxy );
            CORBA::release( tmp_proxy );
            p = "connect_structured_pull_consumer";
            proxy->connect_structured_pull_consumer(
                            CosNotifyComm::StructuredPullConsumer::_nil(), env );
            CORBA::release( admin );

            // ---- Proxy file out ----
            str = orb->object_to_string( proxy, env );
#if defined(_MSC_VER) && (_MSC_VER >= 1400)
            err = fopen_s( &fd, filename, "w" );
            if ( err != 0 ) {
                printf( "file open error %d\n", err );
                return(1);
            }
#else
           fd = fopen( filename, "w" );
            if ( fd == NULL ) {
                printf( "file open error %d\n", errno );
                return(1);
            }
#endif
            fprintf( fd, "%s", str );
            CORBA::string_free(str);
            fclose( fd );
        }

        // ---- create ES_Util interface object ----
        p = "create_util";
        obj= channel->create_util(
                    CosNotifyChannelAdmin::EventChannel::CHANNEL_UTIL, env );
        util = ES::ChannelUtil::_narrow( obj );
        CORBA::release( obj );

        // ---- local begin ----
        p = "local_begin";
        util->local_begin( proxy,
                           env);

        CosNotification::StructuredEvent    *recv;
        try {
            p = "pull_structured_event";
            recv = proxy->pull_structured_event( env );
        } catch( CORBA::SystemException &se ) {
            printf( "Operation=%s, SystemException=%s, minor=%#x\n",
                    p, se.id(), se.minor() );
            util->local_rollback( proxy,
                                  env);
            return(1);
        } catch( CORBA::Exception &e ) {
            printf( "Operation=%s, Exception=%s\n", p, e.id() );
            util->local_rollback( proxy,
                                  env);
            return(1);
        }

        // ---- local commit ----
        p = "local_commit";
        util->local_commit( proxy,
                            env);

        CORBA::Char *message = 
            (CORBA::Char *)recv->remainder_of_body->value();
        printf( "pull message : %s\n", message );
        delete recv;

        CORBA::release( proxy );
        CORBA::release( util );
        CORBA::release( naming_context );
        CORBA::release( channel );

    } catch( CORBA::SystemException   &se ) {
        printf( "Operation=%s, SystemException=%s, minor=%#x\n",
                p, se.id(), se.minor() );
        return(1);
    } catch( CORBA::Exception   &e ) {
        printf( "Operation=%s, Exception=%s\n", p, e.id() );
        return(1);
    }

    return(0);
}

